#!/usr/bin/env bash
# Copyright (c) 2004-2023 Yury V. Vishnevskiy yu.v.vishnevskiy@gmail.com
# All Rights Reserved.
# This file is subject to the terms and conditions defined in the
# 'license.txt' file, which is part of this package.

# This is a script for running UNEX tests.

currentDir=$(pwd)
thisDirName=$(dirname "$0")
cd "$thisDirName" || exit 1          # Running script in its directory, where UNEX root is.
thisDirPath=$(pwd)

function cleanup {
  cd "$currentDir"
}

trap cleanup EXIT


retc=0

# Prefer GNU timeout on non-GNU systems
if [ "$(command -v gtimeout)" ]; then
    tm="gtimeout"
elif [ "$(command -v timeout)" ]; then
    tm="timeout"
else
    tm=""
fi

if [ -n "$tm" ]; then
    # Test --kill-after parameter
    $($tm --kill-after=1s 1s $tm --version > /dev/null 2>&1)
    if [ $? -eq 0 ]; then
	tm="$tm --kill-after=1m 2m"
    else
	tm="$tm 2m"
    fi
fi


if [[ -t 1 ]]; then
    # stdout is a terminal
    GREEN='\033[0;32m'
    RED='\033[0;31m'
    NC='\033[0m' # No Color
else
    # stdout is not a terminal
    GREEN=''
    RED=''
    NC=''
fi


function secs_to_human() {
    num=$1
    min=0
    hour=0
    day=0
    if ((num>59)); then
	((sec=num%60))
	((num=num/60))
	if ((num>59)); then
	    ((min=num%60))
	    ((num=num/60))
	    if ((num>23)); then
		((hour=num%24))
		((day=num/24))
	    else
		((hour=num))
	    fi
	else
	    ((min=num))
	fi
    else
	((sec=num))
    fi
    echo "Time elapsed: ${day}d ${hour}h ${min}m ${sec}s"
}


function check_reference() {
    # $1 -- log file
    # $2 -- ref file
    # $3 -- err file
    local retc=0
    local global_start_line=1
    local cur_start_line=0
    local grepout=""
    # Iterate over lines in the ref file
    while IFS="" read -r refstr || [ -n "$refstr" ]
    do
      # Start grepping from the last matched line.
      grepout=$(tail -n +$global_start_line "$1" | grep -Fin --max-count=1 "$refstr")
      if [ $? -ne 0 ]; then
        retc=$((retc + 1))
        echo "NoRefStr(fromline=$global_start_line): $refstr" >> "$3"
      else
        cur_start_line=$(awk -F: '{print $1}' <<< $grepout)
        global_start_line=$((global_start_line + cur_start_line))
      fi
    done < "$2"
    return $retc
}


TDIR="tests/public/inp"

# Clean up after possible previous tests
delstr=$(find "$TDIR" -type f -not -name "*.inp" -not -name "*.ref" -delete)
delstr=$(find "$TDIR" -type l -not -name "*.inp" -not -name "*.ref" -delete)
delstr=$(find "$TDIR" -type d -empty -delete)


files=$(find "$TDIR" -type f -name "*.inp") || exit 1

ttotal=$(echo "$files" | wc -l)
tcurrent=0
tpassed=0
tpassed_ref=0
tfailed=0
tfailed_ref=0
tskipped=0
ttimeout=0

tstart=$(date +%s)

while IFS= read -r inputf; do
    tcurrent=$((tcurrent + 1))
    printf "%3s of %3s $inputf ... " "$tcurrent" "$ttotal"

    inpdir=$(dirname "$inputf")

    # Prepare filename without extension '.inp'
    fname=$(echo "$inputf" | rev | cut -f 2- -d '.' | rev)
    fname="${fname##*/}"

    ref_used=false

    # Skip some tests if need
    # Ih_C540 is too long.
    if [[ $fname = *"Ih_C540"* ]]; then
        tskipped=$((tskipped + 1))
        printf "Skipped\\n"
        continue
    fi

    # Run test
    $tm ./unex -d "${inpdir}" "${fname}.inp" "${fname}.log" > /dev/null 2> "${inpdir}/${fname}.err"
    retcode=$?

    # Check log file if respective reference file exists
    if [ $retcode -eq 0 ] && [ -f "${inpdir}/${fname}.ref" ]; then
        check_reference "${inpdir}/${fname}.log" "${inpdir}/${fname}.ref" "${inpdir}/${fname}.err"
        retcode=$?
        ref_used=true
    fi

    #Check exit code
    if [ $retcode -eq 0 ]; then
        tpassed=$((tpassed + 1))
        if [ "$ref_used" = true ]; then tpassed_ref=$((tpassed_ref + 1)); fi
        printf "${GREEN}Successful${NC}\\n"
    else
        if [ $retcode -eq 124 ]; then
            ttimeout=$((ttimeout + 1))
            printf "${RED}Timeout${NC}\\n"
        else
            tfailed=$((tfailed + 1))
            if [ "$ref_used" = true ]; then tfailed_ref=$((tfailed_ref + 1)); fi
            printf "${RED}Failed${NC}\\n"
        fi
    fi

    # Remove empty *.err file
    if [ ! -s "${inpdir}/${fname}.err" ]; then rm -f "${inpdir}/${fname}.err"; fi
done <<< "$files"


tend=$(date +%s)


printf "Skipped: ${tskipped}\\n"
printf "Passed:  ${tpassed} (${tpassed_ref} with external reference)\\n"

if [ $tfailed -ne 0 ]; then
    printf "${RED}Failed:  ${tfailed} (${tfailed_ref} with external reference)${NC}\\n"
    retc=1
else
    # Print anyway for clarity
    printf "Failed:  0\\n"
fi

if [ $ttimeout -ne 0 ]; then
    printf "${RED}Timeout: ${ttimeout}${NC}\\n"
    retc=1
else
    # Print anyway for clarity
    printf "Timeout: 0\\n"
fi


secs_to_human "$((tend - tstart))"

exit $retc
